// xcview - xcursor file viewer 1.00
// Copyright (C) 2008, Jason Cipriani (jason dot cipriani at gmail dot com)
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <GL/glut.h>
#include <X11/Xcursor/Xcursor.h>
#include <stdlib.h>
#include <string>
#include <list>
#include <algorithm>

using namespace std;

typedef pair<string,XcursorImage*> NC; // saves me typing
list<NC> g_cursors;
int g_totalh = 0;
int g_offset = 0;
int g_cursorsize = 20;

void reshape (int w, int h) {

  glViewport(0, 0, w, h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0.0, w, h, 0.0, -1.0, 0.0);
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
  glColor3ub(255, 255, 255);

}

void display (void) {

  list<NC>::iterator i;
  int ypos = -g_offset, j;

  glClear(GL_COLOR_BUFFER_BIT);

  for (i = g_cursors.begin(); i != g_cursors.end(); ++ i) {
    XcursorImage *c = (*i).second;
    glRasterPos2i(0, ypos);
    glPixelZoom(1.0f, -1.0f);
    glDrawPixels(c->width, c->height, GL_BGRA, GL_UNSIGNED_BYTE, c->pixels);
    glRasterPos2i(std::max<int>(c->width, 32), ypos + (c->height - 13) / 2 + 13);
    glPixelZoom(1.0f, 1.0f);
    for (j = 0; j < (int)(*i).first.size(); ++ j)
      glutBitmapCharacter(GLUT_BITMAP_8_BY_13, (*i).first[j]);
    ypos += std::max<int>(c->height, 13);
  }

  glutSwapBuffers();

}

void special (int k, int, int) {

  if (k == GLUT_KEY_UP && g_offset > 0)
    g_offset -= g_cursorsize;
  else if (k == GLUT_KEY_DOWN && g_offset < g_totalh)
    g_offset += g_cursorsize;

  glutPostRedisplay();

}

void keyboard (unsigned char ch, int, int) {

  if (ch == 'q' || ch == 'Q' || ch == 27)
    exit(0);

}

const char * my_basename (const char *filename) {

  const char *sep = strrchr(filename, '/');
  return sep ? (sep + 1) : filename;

}

int main (int argc, char **argv) {

  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
  glutInitWindowSize(300, 400);
  glutCreateWindow("XCView");
  glutReshapeFunc(&reshape);
  glutSpecialFunc(&special);
  glutKeyboardFunc(&keyboard);
  glutDisplayFunc(&display);

  for (int a = 1; a < argc; ++ a)
    if (argv[a][0] == '+') {
      g_cursorsize = atoi(argv[a]+1);
      printf("cursor size switched to %i\n", g_cursorsize);
    } else if (argv[a][0] != '-') {
      XcursorImage *c = XcursorFilenameLoadImage(argv[a], g_cursorsize);
      if (c) {
	const char *filename = my_basename(argv[a]);
	printf("%s: %ix%i\n", filename, c->width, c->height);
	g_totalh += c->height;
	g_cursors.push_back(NC(filename, c));
      } else
	fprintf(stderr, "%s: could not load cursor file\n", argv[a]);
    }

  if (g_cursors.empty()) {
    fprintf(stderr, "xcview (c) 2008, jason cipriani\n"
	    "this program comes with ABSOLUTELY NO WARRANTY; it is distributed\n"
	    "under the terms of the Gnu GPL; for more info see COPYING or visit\n"
	    "http://www.gnu.org/licenses.\n\n");
    fprintf(stderr, "usage: %s +cursor_size file [ files ... ]\n\n"
	    "  cursor_size defaults to 20. affects only following files.\n"
	    "  files are xcursor data files.\n"
	    "  use up and down arrows to scroll through list.\n"
	    "  press q or escape or close window to quit.\n\n", argv[0]);
    return 1;
  }

  glutMainLoop();

}
